package com.ruoyi.postCheck.mapper;

import java.util.List;
import com.ruoyi.postCheck.domain.PostPlan;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.ruoyi.postCheck.domain.join.PostPlanJoinChecked;
import com.ruoyi.postCheck.domain.join.PostPlanJoinProblem;

/**
 * 岗检计划Mapper接口
 *
 * @author csg
 * @date 2023-12-19
 */
public interface PostPlanMapper extends BaseMapper<PostPlan>
{
    /**
     * 查询岗检计划
     *
     * @param id 岗检计划主键
     * @return 岗检计划
     */
    public PostPlan selectPostPlanById(Long id);

    /**
     * 查询岗检计划列表
     *
     * @param postPlan 岗检计划
     * @return 岗检计划集合
     */
    public List<PostPlan> selectPostPlanList(PostPlan postPlan);

    /**
     * 新增岗检计划
     *
     * @param postPlan 岗检计划
     * @return 结果
     */
    public int insertPostPlan(PostPlan postPlan);

    /**
     * 修改岗检计划
     *
     * @param postPlan 岗检计划
     * @return 结果
     */
    public int updatePostPlan(PostPlan postPlan);

    /**
     * 删除岗检计划
     *
     * @param id 岗检计划主键
     * @return 结果
     */
    public int deletePostPlanById(Long id);

    /**
     * 批量删除岗检计划
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deletePostPlanByIds(Long[] ids);

    /**
     * 查询岗检问题录入列表列表
     * @param postPlanJoinProblem
     * @return
     */
    List<PostPlanJoinProblem> selectPostPlanJoinProblemList(PostPlanJoinProblem postPlanJoinProblem);

    /**
     * 查询岗检记录上传列表列表
     *
     * @param postPlanJoinChecked 岗检记录上传列表
     * @return 岗检记录上传列表
     */
    List<PostPlanJoinChecked> selectPostPlanJoinCheckedList(PostPlanJoinChecked postPlanJoinChecked);
}
